@@ -108,8 +108,14 @@ namespace FSpot {
 
 			uri_chooser = new Gtk.FileChooserButton (Catalog.GetString ("Select Export Folder"),
 								 Gtk.FileChooserAction.SelectFolder);
-			uri_chooser.SetFilename (uri_path);
+			
 			uri_chooser.LocalOnly = false;
+
+			if (Preferences.Get (Preferences.EXPORT_FOLDER_URI) != null && Preferences.Get (Preferences.EXPORT_FOLDER_URI) as string != String.Empty)
+				uri_chooser.SetUri (Preferences.Get (Preferences.EXPORT_FOLDER_URI) as string);
+			else
+				uri_chooser.SetFilename (uri_path);
+
 			chooser_hbox.PackStart (uri_chooser);
 
 			Dialog.ShowAll ();
@@ -146,7 +152,7 @@ namespace FSpot {
 					gallery_path = Gnome.Vfs.Uri.GetLocalPathFromUri (dest.ToString ());
 
 				progress_dialog.Message = Catalog.GetString ("Building Gallery");
-				progress_dialog.Fraction = 1.0;
+				progress_dialog.Fraction = 0.0;
 
 				FolderGallery gallery;
 				if (static_radio.Active) {
@@ -171,19 +177,44 @@ namespace FSpot {
 
 				gallery.Description = description;
 
-				gallery.Generate ();
+				gallery.GenerateLayout ();
+				Filters.FilterSet filter_set = new Filters.FilterSet ();
+				if (scale)
+					filter_set.Add (new Filters.ResizeFilter ((uint) size));
+				else if (rotate)
+					filter_set.Add (new Filters.OrientationFilter ());
+				filter_set.Add (new Filters.ChmodFilter ());
+				filter_set.Add (new Filters.UniqueNameFilter (gallery_path));
+
+				for (int photo_index = 0; photo_index < selection.Count; photo_index++)
+				{
+					try {	
+						progress_dialog.Message = System.String.Format (Catalog.GetString ("Uploading picture \"{0}\""), selection[photo_index].Name);
+						progress_dialog.Fraction = photo_index / (double) selection.Count;
+						gallery.ProcessImage (photo_index, filter_set);
+						progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of {1}"), photo_index, selection.Count);
+					}
+					catch (Exception e) {
+						progress_dialog.Message = String.Format (Catalog.GetString ("Error uploading picture \"{0}\" to Gallery:\n{1}"), 
+							selection[photo_index].Name, e.Message);
+						progress_dialog.ProgressText = Catalog.GetString ("Error");
+
+						if (progress_dialog.PerformRetrySkip ())
+							photo_index--;
+					}
+		
+				}
 
 				// we've created the structure, now if the destination was local we are done
 				// otherwise we xfer 
 				if (!dest.IsLocal) {
 					Console.WriteLine(target);
-					Gnome.Vfs.XferProgressCallback cb = new Gnome.Vfs.XferProgressCallback (Progress);
 					System.Console.WriteLine ("Xfering {0} to {1}", source.ToString (), target.ToString ());
 					result = Gnome.Vfs.Xfer.XferUri (source, target, 
 									 Gnome.Vfs.XferOptions.Default, 
 									 Gnome.Vfs.XferErrorMode.Abort, 
 									 Gnome.Vfs.XferOverwriteMode.Replace, 
-									 cb);
+									 Progress);
 				}
 
 				if (result == Gnome.Vfs.Result.Ok) {
@@ -208,9 +239,9 @@ namespace FSpot {
 				Preferences.Set (Preferences.EXPORT_FOLDER_OPEN, open);
 				Preferences.Set (Preferences.EXPORT_FOLDER_ROTATE, rotate);
 				Preferences.Set (Preferences.EXPORT_FOLDER_METHOD, static_radio.Active ? "static" : original_radio.Active ? "original" : "folder" );
-
+				Preferences.Set (Preferences.EXPORT_FOLDER_URI, uri_chooser.Uri);
 			} catch (System.Exception e) {
-Console.WriteLine (e);
+				// Console.WriteLine (e);
 				progress_dialog.Message = e.ToString ();
 				progress_dialog.ProgressText = Catalog.GetString ("Error Transferring");
 			} finally {
@@ -353,8 +384,8 @@ Console.WriteLine (e);
 
 		protected ScaleRequest [] requests;
 		
-		protected string [] pixbuf_keys = {"quality"};
-		protected string [] pixbuf_values = {"95"};
+		protected string [] pixbuf_keys = { "quality", null };
+		protected string [] pixbuf_values = { "95", null };
 
 		protected struct ScaleRequest {
 			public string Name;
@@ -389,22 +420,10 @@ Console.WriteLine (e);
 			this.requests = new ScaleRequest [] { ScaleRequest.Default };
 		}
 
-		public virtual void Generate ()
+		public virtual void GenerateLayout ()
 		{
 			MakeDir (gallery_path);
 
-			for (int i = 0; i < collection.Count; i++)
-			{
-				Filters.FilterSet filter_set = new Filters.FilterSet ();
-				if (scale)
-					filter_set.Add (new Filters.ResizeFilter ((uint) size));
-				else if (rotate)
-					filter_set.Add (new Filters.OrientationFilter ());
-				filter_set.Add (new Filters.UniqueNameFilter (gallery_path));
-
-				ProcessImage (i, filter_set);
-
-			}
 		}
 
 		protected virtual string ImageName (int image_num)
@@ -416,13 +435,13 @@ Console.WriteLine (e);
 		{
 			IBrowsableItem photo = collection [image_num];
 			string photo_path = photo.DefaultVersionUri.LocalPath;
-//			string path;
+			string path;
 			ScaleRequest req;
 
 			req = requests [0];
 			
 			MakeDir (SubdirPath (req.Name));
-//			path = SubdirPath (req.Name, ImageName (image_num));
+			path = SubdirPath (req.Name, ImageName (image_num));
 			
 //			if (scale) {
 //				//FIXME we have filters for that !
@@ -435,49 +454,54 @@ Console.WriteLine (e);
 
 			using (Filters.FilterRequest request = new Filters.FilterRequest (photo.DefaultVersionUri)) {
 				filter_set.Convert (request);
-				request.Preserve (request.Current);
-				//			File.Copy (request.Current.LocalPath, System.IO.Path.Combine(SubdirPath (req.Name), System.IO.Path.GetFileName (request.Current.LocalPath)), false);
+				if (request.Current.LocalPath == path)
+					request.Preserve(request.Current);
+				else
+					File.Copy (request.Current.LocalPath, path, true); 
 
 				if (photo != null && photo is Photo && Core.Database != null) {
 					Core.Database.Exports.Create ((photo as Photo).Id, (photo as Photo).DefaultVersionId,
 								      ExportStore.FolderExportType,
-								      request.Current.ToString ());
+								      // FIXME this is wrong, the final path is the one
+								      // after the Xfer.
+								      UriList.PathToFileUriEscaped (path).ToString ());
 				}
 
 				
 				Gdk.Pixbuf img = null;
 				Gdk.Pixbuf scaled = null;
 				
-				Exif.ExifData data = new Exif.ExifData (photo_path);
-				if (data != null && data.Handle.Handle == System.IntPtr.Zero)
-					data = null;
-				
-				for (int i = 1; i < requests.Length; i++) {
-					string path;
+				using (Exif.ExifData data = new Exif.ExifData (photo_path)) {
+					for (int i = 1; i < requests.Length; i++) {
+						
+						req = requests [i];
+						if (scale && req.AvoidScale (size))
+							continue;
 					
-					req = requests [i];
-					if (scale && req.AvoidScale (size))
-						continue;
+						if (img == null) {
+							ImageFile imgf = ImageFile.Create (photo.DefaultVersionUri);
+							scaled = imgf.Load (req.Width, req.Height);
+						} else
+							scaled = PixbufUtils.ScaleToMaxSize (img, req.Width, req.Height, false);
+						
+						MakeDir (SubdirPath (req.Name));
+						path = SubdirPath (req.Name, ImageName (image_num));
+						
+						if (req.CopyExif && data != null && data.Handle.Handle == System.IntPtr.Zero) {
+							PixbufUtils.SaveJpeg (scaled, path, 90, data);
+						} else 
+							scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
+						
+						if (img != null)
+							img.Dispose ();
+						
+						img = scaled;
+					}
 					
-					if (img == null) 
-						scaled = PixbufUtils.LoadAtMaxSize (photo_path, req.Width, req.Height);
-					else
-						scaled = PixbufUtils.ScaleToMaxSize (img, req.Width, req.Height, false);
-					
-					MakeDir (SubdirPath (req.Name));
-					path = SubdirPath (req.Name, ImageName (image_num));
-					
-					if (req.CopyExif && data != null) {
-						PixbufUtils.SaveJpeg (scaled, path, 90, data);
-					} else 
-						scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
-					
-					if (img != null)
-						img.Dispose ();
+					if (scaled != null)
+						scaled.Dispose ();
+
 				}
-				
-				if (scaled != null)
-					scaled.Dispose ();
 			}
 		}
 		
@@ -577,9 +601,9 @@ Console.WriteLine (e);
 							 new ScaleRequest ("thumbs", 120, 120, false) };
 		}
 
-		public override void Generate ()
+		public override void GenerateLayout ()
 		{
-			base.Generate ();
+			base.GenerateLayout ();
 			MakeDir (SubdirPath ("comments"));
 			MakeDir (SubdirPath ("zip"));
 			CreateHtaccess();
@@ -689,8 +713,8 @@ Console.WriteLine (e);
 		string altstylesheet = "f-spot-simple-white.css";
 		string javascript = "f-spot.js";
 
-        static string light = Catalog.GetString("Light");
-        static string dark = Catalog.GetString("Dark");
+		static string light = Catalog.GetString("Light");
+		static string dark = Catalog.GetString("Dark");
 		
 		public HtmlGallery (IBrowsableCollection selection, string path, string name) : base (selection, path, name) 
 		{ 
@@ -704,12 +728,12 @@ Console.WriteLine (e);
 			return String.Format ("img-{0}.jpg", photo_index + 1);
 		}
 
-		public override void Generate ()
+		public override void GenerateLayout ()
 		{
 			if (collection.Count == 0)
 				return;
 			
-			base.Generate ();
+			base.GenerateLayout ();
 			
 			IBrowsableItem [] photos = collection.Items;
 			
@@ -722,39 +746,42 @@ Console.WriteLine (e);
 			
 			MakeDir (SubdirPath ("style"));
 			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetCallingAssembly ();
-			System.IO.Stream s = assembly.GetManifestResourceStream (stylesheet);
-			System.IO.Stream fs = System.IO.File.Open (SubdirPath ("style", stylesheet), System.IO.FileMode.Create);
+			using (Stream s = assembly.GetManifestResourceStream (stylesheet)) {
+				using (Stream fs = System.IO.File.Open (SubdirPath ("style", stylesheet), System.IO.FileMode.Create)) {
 
-			byte [] buffer = new byte [8192];
-			int n;
-			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
-				fs.Write (buffer, 0,  n);						    
-			
-			fs.Close ();
+					byte [] buffer = new byte [8192];
+					int n;
+					while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+						fs.Write (buffer, 0,  n);						    
+					
+				}
+			}
 			/* quick and stupid solution
 			   this should have been iterated over an array of stylesheets, really
 			*/
-			s = assembly.GetManifestResourceStream (altstylesheet);
-			fs = System.IO.File.Open (SubdirPath ("style", altstylesheet), System.IO.FileMode.Create);
+			using (Stream s = assembly.GetManifestResourceStream (altstylesheet)) {
+				using (Stream fs = System.IO.File.Open (SubdirPath ("style", altstylesheet), System.IO.FileMode.Create)) {
+					
+					byte [] buffer = new byte [8192];
+					int n = 0;
+					while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+						fs.Write (buffer, 0,  n);						    
+					
+				}
+			}
 
-			buffer = new byte [8192];
-			n = 0;
-			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
-				fs.Write (buffer, 0,  n);						    
-			
-			fs.Close ();
 			/* Javascript for persistant style change */
 			MakeDir (SubdirPath ("script"));
-			s = assembly.GetManifestResourceStream (javascript);
-			fs = System.IO.File.Open (SubdirPath ("script", javascript), System.IO.FileMode.Create);
+			using (Stream s = assembly.GetManifestResourceStream (javascript)) {
+				using (Stream fs = System.IO.File.Open (SubdirPath ("script", javascript), System.IO.FileMode.Create)) {
 
-			buffer = new byte [8192];
-			n = 0;
-			while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
-				fs.Write (buffer, 0,  n);						    
-			
-			fs.Close ();
-	
+					byte [] buffer = new byte [8192];
+					int n = 0;
+					while ((n = s.Read (buffer, 0, buffer.Length)) != 0)
+						fs.Write (buffer, 0,  n);						    
+					
+				}
+			}
 		}
 		
 		public int PageCount {
