@@ -7,6 +7,8 @@ using System.Collections.Specialized;
 using System.Web;
 using Mono.Unix;
 
+using FSpot.Filters;
+
 using GalleryRemote;
 
 namespace FSpot {
@@ -45,11 +47,10 @@ namespace FSpot {
 
 			System.Console.WriteLine ("Gallery created: " + gal);
 
-			if (gal.Login (username, password)) {
-				//System.Console.WriteLine ("Login successful");
-				gallery = gal;
-				connected = true;
-			}
+			gal.Login (username, password);
+
+			gallery = gal;
+			connected = true;
 
 			return gallery;
 		}
@@ -356,8 +357,17 @@ namespace FSpot {
 					if (uri.Scheme != Uri.UriSchemeHttp &&
 					    uri.Scheme != Uri.UriSchemeHttps)
 						throw new System.UriFormatException ();
-
-				} catch (System.UriFormatException e) {
+					
+					GalleryAccount created = new GalleryAccount (name, 
+										     url, 
+										     username,
+										     password);
+					
+					created.Connect ();
+					GalleryAccountManager.GetInstance ().AddAccount (created);
+					account = created;
+				} catch (System.UriFormatException) {
+					
 					HigMessageDialog md = 
 						new HigMessageDialog (Dialog, 
 								      Gtk.DialogFlags.Modal |
@@ -368,13 +378,33 @@ namespace FSpot {
 					md.Run ();
 					md.Destroy ();
 					return;
+				} catch (GalleryRemote.GalleryException e) {
+					HigMessageDialog md = 
+						new HigMessageDialog (Dialog, 
+								      Gtk.DialogFlags.Modal |
+								      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+								      Catalog.GetString ("Error while connecting to Gallery"),
+								      String.Format (Catalog.GetString ("The following error was encountered while attempting to log in: {0}"), e.Message));
+					if (e.ResponseText != null) {
+						System.Console.WriteLine (e.Message);
+						System.Console.WriteLine (e.ResponseText);
+					}
+					md.Run ();
+					md.Destroy ();
+					return;
+				} catch (System.Net.WebException we) {
+					HigMessageDialog md = 
+						new HigMessageDialog (Dialog, 
+								      Gtk.DialogFlags.Modal |
+								      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+								      Catalog.GetString ("Error while connecting to Gallery"),
+								      String.Format (Catalog.GetString ("The following error was encountered while attempting to log in: {0}"), we.Message));
+					md.Run ();
+					md.Destroy ();
+					return;
 				}
-
-				GalleryAccount account = new GalleryAccount (name, 
-									     url, 
-									     username,
-									     password);
-				GalleryAccountManager.GetInstance ().AddAccount (account);
 			}
 			Dialog.Destroy ();
 		}
@@ -448,7 +478,7 @@ namespace FSpot {
 		{
 			Gtk.Menu menu = new Gtk.Menu ();
 			if (gallery.Version == GalleryVersion.Version1) {
-				Gtk.MenuItem top_item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(TopLevel)"));
+				Gtk.MenuItem top_item = new Gtk.MenuItem (Catalog.GetString ("(TopLevel)"));
 				menu.Append (top_item);
 			}
 			
@@ -477,10 +507,18 @@ namespace FSpot {
 		
 		private void HandleChanged (object sender, EventArgs args)
 		{
-			if (gallery.Albums.Count == 0 || album_optionmenu.History <= 0) {
-				parent = "";
+			if (gallery.Version == GalleryVersion.Version1) {
+				if (gallery.Albums.Count == 0 || album_optionmenu.History <= 0) {
+					parent = "";
+				} else {
+					parent = ((Album) gallery.Albums [album_optionmenu.History-1]).Name;
+				}
 			} else {
-				parent = ((Album) gallery.Albums [album_optionmenu.History-1]).Name;
+				if (gallery.Albums.Count == 0 || album_optionmenu.History < 0) {
+					parent = "";
+				} else {
+					parent = ((Album) gallery.Albums [album_optionmenu.History]).Name;
+				}
 			}
 			name = name_entry.Text;
 			description = description_entry.Text;
@@ -507,7 +545,7 @@ namespace FSpot {
 	public class GalleryExport : GladeDialog {
 		public GalleryExport (IBrowsableCollection selection) : base ("gallery_export_dialog")
 		{
-			this.photos = (Photo []) selection.Items;
+			this.items = selection.Items;
 			album_button.Sensitive = false;
 			IconView view = new IconView (selection);
 			view.DisplayDates = false;
@@ -528,7 +566,6 @@ namespace FSpot {
 			if (edit_button != null)
 				edit_button.Clicked += HandleEditGallery;
 			
-			rh = new Gtk.ResponseHandler (HandleResponse);
 			Dialog.Response += HandleResponse;
 			connect = true;
 			HandleSizeActive (null, null);
@@ -541,8 +578,6 @@ namespace FSpot {
 			LoadPreference (Preferences.EXPORT_GALLERY_ROTATE);
 		}
 		
-		Gtk.ResponseHandler rh;
-		
 		private bool scale;
 		private bool rotate;
 		private int size;
@@ -550,7 +585,7 @@ namespace FSpot {
 		private bool meta;
 		private bool connect = false;
 
-		Photo [] photos;
+		IBrowsableItem[] items;
 		int photo_index;
 		FSpot.ThreadProgressDialog progress_dialog;
 		
@@ -560,10 +595,6 @@ namespace FSpot {
 
 		private string xml_path;
 
-		// Dialogs
-		private AccountDialog gallery_add;
-		private GalleryAddAlbum album_add;
-
 		// Widgets
 		[Glade.Widget] Gtk.OptionMenu gallery_optionmenu;
 		[Glade.Widget] Gtk.OptionMenu album_optionmenu;
@@ -615,9 +646,9 @@ namespace FSpot {
 				Dialog.Destroy ();
 
 				command_thread = new System.Threading.Thread (new System.Threading.ThreadStart (this.Upload));
-				command_thread.Name = Mono.Posix.Catalog.GetString ("Uploading Pictures");
+				command_thread.Name = Catalog.GetString ("Uploading Pictures");
 				
-				progress_dialog = new FSpot.ThreadProgressDialog (command_thread, photos.Length);
+				progress_dialog = new FSpot.ThreadProgressDialog (command_thread, items.Length);
 				progress_dialog.Start ();
 
 				// Save these settings for next time
@@ -632,7 +663,7 @@ namespace FSpot {
 		private void HandleProgressChanged (ProgressItem item)
 		{
 			//System.Console.WriteLine ("Changed value = {0}", item.Value);
-			progress_dialog.Fraction = (photo_index - 1.0 + item.Value) / (double) photos.Length;
+			progress_dialog.Fraction = (photo_index - 1.0 + item.Value) / (double) items.Length;
 		}
 
 		public void HandleSizeActive (object sender, EventArgs args)
@@ -640,59 +671,58 @@ namespace FSpot {
 			size_spin.Sensitive = scale_check.Active;
 		}
 
+	
 		private void Upload ()
 		{
-			try {
 				account.Gallery.Progress = new ProgressItem ();
 				account.Gallery.Progress.Changed += HandleProgressChanged;
 
 				System.Console.WriteLine ("Starting upload");
-				while (photo_index < photos.Length) {
-					Photo photo = photos [photo_index];
+				
+				FilterSet filters = new FilterSet ();
+				if (scale)
+					filters.Add (new ResizeFilter ((uint) size));
+				else if (rotate)
+					filters.Add (new OrientationFilter ());
+				
+
+				while (photo_index < items.Length) {
+					IBrowsableItem item = items [photo_index];
 
 					System.Console.WriteLine ("uploading {0}", photo_index);
 
-					progress_dialog.Message = System.String.Format (Mono.Posix.Catalog.GetString ("Uploading picture \"{0}\""), photo.Name);
-					progress_dialog.Fraction = photo_index / (double) photos.Length;
+					progress_dialog.Message = System.String.Format (Catalog.GetString ("Uploading picture \"{0}\""), item.Name);
+					progress_dialog.Fraction = photo_index / (double) items.Length;
 					photo_index++;
 
-					progress_dialog.ProgressText = System.String.Format (Mono.Posix.Catalog.GetString ("{0} of {1}"), photo_index, photos.Length);
+					progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of {1}"), photo_index, items.Length);
 					
-					if (scale) {
-						/* FIXME this is sick */
-						string orig = photo.DefaultVersionUri.LocalPath;
-						string path = PixbufUtils.Resize (orig, size, true);
-						string final = path + System.IO.Path.GetExtension (orig);
-						System.IO.File.Move (path, final);
-						album.Add (photo, final);
-						System.IO.File.Delete (final);
-					} else if (rotate) {
-						string orig = photo.DefaultVersionUri.LocalPath;
-						// fixme hack to trick the stupid code in ImageFile.Create
-						string final = ImageFile.TempPath (orig);
+					
+					Filters.FilterRequest req = new Filters.FilterRequest (item.DefaultVersionUri);
 
-						if (OrientationFilter.Convert (orig, final)) {
-							album.Add (photo, final);
-							System.IO.File.Delete (final);
-						} else
-							album.Add (photo);
+					filters.Convert (req);
+				try {
+					int id = album.Add (item, req.Current.LocalPath);
 
-						System.IO.File.Delete (final);
-					} else {
-						album.Add (photo);
+					if (item != null && item is Photo && Core.Database != null && id != 0) {
+						Core.Database.Exports.Create ((item as Photo).Id, (item as Photo).DefaultVersionId,
+									      ExportStore.Gallery2ExportType,
+									      String.Format("{0}:{1}",album.Gallery.Uri.ToString (), id.ToString ()));
+					}
+				} catch (System.Exception e) {
+					progress_dialog.Message = String.Format (Catalog.GetString ("Error uploading picture \"{0}\" to Gallery: {1}"), item.Name, e.Message);
+					progress_dialog.ProgressText = Catalog.GetString ("Error");
+
+					if (progress_dialog.PerformRetrySkip ()) {
+						photo_index--;
 					}
 				}
-
-				progress_dialog.Message = Mono.Posix.Catalog.GetString ("Done Sending Photos");
-				progress_dialog.Fraction = 1.0;
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Upload Complete");
-				progress_dialog.ButtonLabel = Gtk.Stock.Ok;
-			} catch (System.Exception e) {
-				progress_dialog.Message = String.Format (Mono.Posix.Catalog.GetString ("Error Uploading To Gallery: {0}"),
-									 e.Message);
-				progress_dialog.ProgressText = Mono.Posix.Catalog.GetString ("Error");
-				System.Console.WriteLine (e);
 			}
+
+			progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
+			progress_dialog.Fraction = 1.0;
+			progress_dialog.ProgressText = Catalog.GetString ("Upload Complete");
+			progress_dialog.ButtonLabel = Gtk.Stock.Ok;
 			
 			if (browser) {
 				GnomeUtil.UrlShow (null, album.GetUrl());
@@ -707,7 +737,7 @@ namespace FSpot {
 
 			accounts = manager.GetAccounts ();
 			if (accounts == null || accounts.Count == 0) {
-				Gtk.MenuItem item = new Gtk.MenuItem (Mono.Posix.Catalog.GetString ("(No Gallery)"));
+				Gtk.MenuItem item = new Gtk.MenuItem (Catalog.GetString ("(No Gallery)"));
 				menu.Append (item);
 				gallery_optionmenu.Sensitive = false;
 				edit_button.Sensitive = false;
@@ -758,12 +788,7 @@ namespace FSpot {
 				PopulateAlbumOptionMenu (account.Gallery);
 				album_button.Sensitive = false;
 				
-				AccountDialog dialog = new AccountDialog (this.Dialog, account, true);
-				Gtk.ResponseType response = (Gtk.ResponseType) dialog.Dialog.Run ();
-
-				if (response == Gtk.ResponseType.Ok) {
-					Connect (account);					
-				}
+				new AccountDialog (this.Dialog, account, true);
 			} 
 		}
 
@@ -799,8 +824,8 @@ namespace FSpot {
 			bool disconnected = gallery == null || !account.Connected || albums == null;
 
 			if (disconnected || albums.Count == 0) {
-				string msg = disconnected ? Mono.Posix.Catalog.GetString ("(Not Connected)") 
-					: Mono.Posix.Catalog.GetString ("(No Albums)");
+				string msg = disconnected ? Catalog.GetString ("(Not Connected)") 
+					: Catalog.GetString ("(No Albums)");
 
 				Gtk.MenuItem item = new Gtk.MenuItem (msg);
 				menu.Append (item);
@@ -830,7 +855,7 @@ namespace FSpot {
 						item.Sensitive = false;
 				}
 
-				ok_button.Sensitive = photos.Length > 0;
+				ok_button.Sensitive = items.Length > 0;
 				album_optionmenu.Sensitive = true;
 				album_button.Sensitive = true;
 			}
@@ -841,12 +866,12 @@ namespace FSpot {
 		
 		public void HandleAddGallery (object sender, System.EventArgs args)
 		{
-			gallery_add = new AccountDialog (this.Dialog);
+			new AccountDialog (this.Dialog);
 		}
 		
 		public void HandleEditGallery (object sender, System.EventArgs args)
 		{
-			gallery_add = new AccountDialog (this.Dialog, account, false);
+			new AccountDialog (this.Dialog, account, false);
 		}
 
 		public void HandleAddAlbum (object sender, System.EventArgs args)
@@ -854,7 +879,7 @@ namespace FSpot {
 			if (account == null)
 				throw new Exception (Catalog.GetString ("No account selected"));
 				
-			album_add = new GalleryAddAlbum (this, account.Gallery);
+			new GalleryAddAlbum (this, account.Gallery);
 		}
 
 		void LoadPreference (string key)
