@@ -113,6 +113,17 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
+	private bool display_filenames = false;
+	public bool DisplayFilenames {
+		get { return display_filenames; }
+		set {
+			if (value != display_filenames) {
+				display_filenames = value;
+				QueueResize ();
+			}
+		}
+	}
+
 	// Size of the frame around the thumbnail.
 	protected int cell_border_width = 10;
 
@@ -128,7 +139,7 @@ public class IconView : Gtk.Layout {
 	// Horizontal spacing between the tag icons
 	protected int tag_icon_hspacing = 2;
 
-	// Vertical spacing between the thumbnail and the row of tag icons.
+	// Vertical spacing between the thumbnail and additional infos (tags, dates, ...).
 	protected int tag_icon_vspacing = 3;
 
 	// Various other layout values.
@@ -142,14 +153,13 @@ public class IconView : Gtk.Layout {
 	private int y_offset;
 	private int x_offset;
 
-	// Drag and drop bookkeeping. 
-	private int click_x, click_y;
-
 	// Focus Handling
 	private int real_focus_cell;
 	public int FocusCell {
 		set {
 			if (value != real_focus_cell) {
+				value = Math.Max (value, 0);
+				value = Math.Min (value, collection.Count - 1);
 				InvalidateCell (value);
 				InvalidateCell (real_focus_cell);
 				real_focus_cell = value;
@@ -164,9 +174,6 @@ public class IconView : Gtk.Layout {
 	// about.
 	private int click_count;
 
-	// The pixbuf we use when we can't load a thumbnail.
-	static Pixbuf error_pixbuf;
-
 	// Public events.
 	public delegate void DoubleClickedHandler (IconView view, int clicked_item);
 	public event DoubleClickedHandler DoubleClicked;
@@ -197,7 +204,7 @@ public class IconView : Gtk.Layout {
 		
 		CanFocus = true;
 
-		FSpot.Global.ModifyColors (this);
+		//FSpot.Global.ModifyColors (this);
 	}
 	
 	public IconView (FSpot.IBrowsableCollection collection) : this () 
@@ -205,6 +212,7 @@ public class IconView : Gtk.Layout {
 		this.collection = collection;
 		this.selection = new SelectionCollection (collection);
 		
+		Name = "ImageContainer";
 		collection.Changed += HandleChanged;
 		collection.ItemsChanged += HandleItemsChanged;
 
@@ -297,6 +305,13 @@ public class IconView : Gtk.Layout {
 				DetailedChanged (this, null);
 
 		}
+		
+		public void MarkChanged (int item)
+		{
+			// Forward the change event up to our parent
+			// we'll fire the event when the parent calls us back
+			parent.MarkChanged ((int) selected_cells [item]);
+		}
 
 		private void HandleParentItemsChanged (IBrowsableCollection collection, BrowsableArgs args)
 		{			
@@ -404,6 +419,9 @@ public class IconView : Gtk.Layout {
 
 		private void Add (int num, bool notify)
 		{
+			if (num == -1)
+				return;
+		       
 			if (this.Contains (num))
 			    return;
 			
@@ -508,7 +526,7 @@ public class IconView : Gtk.Layout {
 			selection = null;
 			items = null;
 		}
-
+		
 		private void SignalChange (int [] ids) 
 		{
 			ClearCached ();
@@ -627,8 +645,11 @@ public class IconView : Gtk.Layout {
 		cells_per_row = Math.Max ((int) (available_width / cell_width), 1);
 		cell_width += (available_width - cells_per_row * cell_width) / cells_per_row;
 		
+		if (DisplayTags || DisplayDates || DisplayFilenames)
+			cell_height += tag_icon_vspacing;
+
 		if (DisplayTags)
-			cell_height += tag_icon_size + tag_icon_vspacing;
+			cell_height += tag_icon_size;
 		
 		if (DisplayDates && this.Style != null) {
 			Pango.FontMetrics metrics = this.PangoContext.GetMetrics (this.Style.FontDescription, 
@@ -636,6 +657,12 @@ public class IconView : Gtk.Layout {
 			cell_height += PangoPixels (metrics.Ascent + metrics.Descent);
 		}
 
+		if (DisplayFilenames && this.Style != null) {
+			Pango.FontMetrics metrics = this.PangoContext.GetMetrics (this.Style.FontDescription,
+										Pango.Language.FromString ("en_US"));
+			cell_height += PangoPixels (metrics.Ascent + metrics.Descent);
+		}
+
 		int num_thumbnails;
 		if (collection != null)
 			num_thumbnails = collection.Count;
@@ -733,12 +760,13 @@ public class IconView : Gtk.Layout {
 			entry.Data = thumbnail_num;
 
 		bool selected = selection.Contains (thumbnail_num);
-		StateType cell_state = selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal;
+		StateType cell_state = selected ? (HasFocus ? StateType.Selected : StateType.Active) : State;
 		
-		Style.PaintFlatBox (Style, BinWindow, cell_state, 
-				    ShadowType.Out, area, this, "IconView", 
-				    bounds.X, bounds.Y,
-				    bounds.Width - 1, bounds.Height - 1);
+		if (cell_state != State)
+			Style.PaintBox (Style, BinWindow, cell_state, 
+					ShadowType.Out, area, this, "IconView", 
+					bounds.X, bounds.Y,
+					bounds.Width - 1, bounds.Height - 1);
 		
 		Gdk.Rectangle focus = Gdk.Rectangle.Inflate (bounds, -3, -3);
 
@@ -785,11 +813,9 @@ public class IconView : Gtk.Layout {
 									      
 					
 					lock (entry) {
-						if (entry.Reload && expansion == 0) {
-							if (entry.Path != null) {
-								entry.Pixbuf = PixbufUtils.ShallowCopy (temp_thumbnail);
-								entry.Reload = true;
-							}
+						if (entry.Reload && expansion == 0 && !entry.IsDisposed) {
+							entry.SetPixbufExtended (PixbufUtils.ShallowCopy (temp_thumbnail), false);
+							entry.Reload = true;
 						}
 					}
 				} else {
@@ -853,7 +879,35 @@ public class IconView : Gtk.Layout {
 			
 			layout.GetPixelSize (out layout_bounds.Width, out layout_bounds.Height);
 
-			layout_bounds.Y = bounds.Y + bounds.Height - cell_border_width - layout_bounds.Height;
+			layout_bounds.Y = bounds.Y + bounds.Height - cell_border_width - layout_bounds.Height + tag_icon_vspacing;
+			layout_bounds.X = bounds.X + (bounds.Width - layout_bounds.Width) / 2;
+			
+			if (DisplayTags)
+				layout_bounds.Y -= tag_icon_size;
+
+			if (DisplayFilenames) {
+				Pango.FontMetrics metrics = this.PangoContext.GetMetrics (this.Style.FontDescription,
+						Pango.Language.FromString ("en_US"));
+				layout_bounds.Y -= PangoPixels (metrics.Ascent + metrics.Descent); 
+			}
+
+			if (layout_bounds.Intersect (area, out region)) {
+				Style.PaintLayout (Style, BinWindow, cell_state,
+						   true, area, this, "IconView", 
+						   layout_bounds.X, layout_bounds.Y, 
+						   layout);
+			}
+		}
+
+		if (DisplayFilenames) {
+
+			string filename = System.IO.Path.GetFileName (photo.DefaultVersionUri.LocalPath);
+			Pango.Layout layout = new Pango.Layout (this.PangoContext);
+			layout.SetText (filename);
+			
+			layout.GetPixelSize (out layout_bounds.Width, out layout_bounds.Height);
+
+			layout_bounds.Y = bounds.Y + bounds.Height - cell_border_width - layout_bounds.Height + tag_icon_vspacing;
 			layout_bounds.X = bounds.X + (bounds.Width - layout_bounds.Width) / 2;
 			
 			if (DisplayTags)
@@ -865,14 +919,15 @@ public class IconView : Gtk.Layout {
 						   layout_bounds.X, layout_bounds.Y, 
 						   layout);
 			}
+			
 		}
 
 		if (DisplayTags) {
 			Tag [] tags = photo.Tags;
 			Gdk.Rectangle tag_bounds;
 
-			tag_bounds.X = bounds.X + (bounds.Width  + tag_icon_vspacing - tags.Length * (tag_icon_size + tag_icon_vspacing)) / 2;
-			tag_bounds.Y = bounds.Y + bounds.Height - cell_border_width - tag_icon_size;
+			tag_bounds.X = bounds.X + (bounds.Width  + tag_icon_hspacing - tags.Length * (tag_icon_size + tag_icon_hspacing)) / 2;
+			tag_bounds.Y = bounds.Y + bounds.Height - cell_border_width - tag_icon_size + tag_icon_vspacing;
 			tag_bounds.Width = tag_icon_size;
 			tag_bounds.Height = tag_icon_size;
 			
@@ -880,18 +935,17 @@ public class IconView : Gtk.Layout {
 				if (t == null)
 					continue;
 
-				Pixbuf icon = null;
-				if (t.Category.Icon == null) {
-					if (t.Icon == null)
-						continue;
-					icon = t.Icon;
-				} else {
-					Category category = t.Category;
-					while (category.Category.Icon != null)
-						category = category.Category;
-					icon = category.Icon;
-				}
+				Pixbuf icon = t.Icon;
 
+		                Tag tag_iter = t.Category;
+		                while (icon == null && tag_iter != Core.Database.Tags.RootCategory && tag_iter != null) {
+		                    icon = tag_iter.Icon;
+		                    tag_iter = tag_iter.Category;
+		                }
+		
+		                if (icon == null)
+		                    continue;
+		
 				if (tag_bounds.Intersect (area, out region)) {
 					Pixbuf scaled_icon;
 					if (icon.Width == tag_bounds.Width) {
@@ -912,9 +966,10 @@ public class IconView : Gtk.Layout {
 						scaled_icon.Dispose ();
 					}
 				}
-				tag_bounds.X += tag_bounds.Width + tag_icon_vspacing;
+				tag_bounds.X += tag_bounds.Width + tag_icon_hspacing;
 			}
 		}
+
 	}
 
 	private void DrawAllCells (Gdk.Rectangle area)
@@ -1189,17 +1244,17 @@ public class IconView : Gtk.Layout {
 		Gdk.Pixbuf result = entry.ShallowCopyPixbuf ();
 		int order = (int) entry.Data;
 
-		if (order > 0 && order < collection.Count) {
+		if (order >= 0 && order < collection.Count) {
 			System.Uri uri = collection [order].DefaultVersionUri;
 
 			if (result == null && !System.IO.File.Exists (FSpot.ThumbnailGenerator.ThumbnailPath (uri)))
-				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
+				FSpot.ThumbnailGenerator.Default.Request (uri, 0, 256, 256);
 			
 			if (result == null)
 				return;
 			
 			if (!FSpot.PhotoLoader.ThumbnailIsValid (uri, result))
-				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
+				FSpot.ThumbnailGenerator.Default.Request (uri, 0, 256, 256);
 		}
 			
 		if (result == null)
@@ -1279,6 +1334,32 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
+	private void SetColors () 
+	{
+		if (IsRealized) {
+			BinWindow.Background = Style.BaseColors [(int)State];
+			GdkWindow.Background = Style.BaseColors [(int)State];
+		}
+	}
+
+	protected override void OnRealized ()
+	{
+		base.OnRealized ();
+		SetColors ();
+	}
+
+	protected override void OnStateChanged (StateType previous)
+	{
+		base.OnStateChanged (previous);
+		SetColors ();
+	}
+	
+	protected override void OnStyleSet (Style previous)
+	{
+		base.OnStyleSet (previous);
+		SetColors ();
+	}
+
 	protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 	{
 		scroll_value = (Vadjustment.Value)/ (Vadjustment.Upper);
@@ -1337,9 +1418,6 @@ public class IconView : Gtk.Layout {
 			if (args.Event.Button != 1)
 				return;
 
-			click_x = (int) args.Event.X;
-			click_y = (int) args.Event.Y;
-
 			FocusCell = cell_num;
 			return;
 
@@ -1392,21 +1470,29 @@ public class IconView : Gtk.Layout {
 		focus_old = FocusCell;
 		switch (args.Event.Key) {
 		case Gdk.Key.Down:
+		case Gdk.Key.J:
+		case Gdk.Key.j:
 			FocusCell += cells_per_row;
 			break;
 		case Gdk.Key.Left:
+		case Gdk.Key.H:
+		case Gdk.Key.h:
 			if (control && shift)
 				FocusCell -= FocusCell % cells_per_row;
 			else
 				FocusCell--;
 			break;
 		case Gdk.Key.Right:
+		case Gdk.Key.L:
+		case Gdk.Key.l:
 			if (control && shift)
 				FocusCell += cells_per_row - (FocusCell % cells_per_row) - 1;
 			else
 				FocusCell++;
 			break;
 		case Gdk.Key.Up:
+		case Gdk.Key.K:
+		case Gdk.Key.k:
 			FocusCell -= cells_per_row;
 			break;
 		case Gdk.Key.Home:
@@ -1421,8 +1507,6 @@ public class IconView : Gtk.Layout {
 		case Gdk.Key.Return:
 			if (DoubleClicked == null)
 				break;
-			if (FocusCell < 0 || FocusCell > collection.Count - 1)
-				break;
 			DoubleClicked (this, FocusCell);
 			break;
 		default:	
@@ -1430,9 +1514,6 @@ public class IconView : Gtk.Layout {
 			return;		
 		}
 		
-		FocusCell = Math.Max (FocusCell, 0);
-		FocusCell = Math.Min (FocusCell, collection.Count - 1);
-		
 		if (FocusCell == focus_old) {
 			args.RetVal = false;
 			return;
